package com.flyer.foster.schedule;

import com.flyer.foster.consts.WechatConst;
import com.flyer.foster.dto.app.AppSubscribeRespDTO;
import com.flyer.foster.entity.AppSubscribe;
import com.flyer.foster.service.IAppSubscribeService;
import com.flyer.foster.util.WeChatApiUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * SendSubscribeMsg
 *
 * @author kelei
 * @since 2024/5/17/9:24
 */
@Slf4j
@Component
public class SendSubscribeMsgSchedule {
    @Autowired
    private IAppSubscribeService iAppSubscribeService;

    @Autowired
    private WeChatApiUtil weChatApiUtil;

    /**
     * 每天上午9点、晚上9点发送订阅消息
     */
    @Scheduled(cron = "0 0 9,21 * * ?")
    public void execute() {
        log.info("begin SendSubscribeMsgSchedule--->execute");
        String accessToken = weChatApiUtil.getAccessToken(WechatConst.APP_ID, WechatConst.APP_SECRET);

        // 获取已经订阅用户信息
        List<AppSubscribeRespDTO> subscribeUserInfoList = iAppSubscribeService.getSubscribeUserInfo();
        for (AppSubscribeRespDTO respDTO : subscribeUserInfoList) {
            int count = respDTO.getCount();
            // 订阅次数-1
            iAppSubscribeService
                    .lambdaUpdate()
                    .set(AppSubscribe::getCount, --count)
                    .eq(AppSubscribe::getAppUserId, respDTO.getAppUserId())
                    .update();
            // 给订阅用户发消息
            weChatApiUtil.subscribeMessage(accessToken, respDTO.getOpenId(), WechatConst.SUBSCRIBE_TEMPLATE1);
        }
        log.info("end SendSubscribeMsgSchedule--->execute");
    }
}
