package com.flyer.foster.service.impl;

import com.flyer.exception.BusinessException;
import com.flyer.foster.dto.RoleMenuAddDTO;
import com.flyer.foster.entity.RoleMenu;
import com.flyer.foster.mapper.IRoleMenuMapper;
import com.flyer.foster.service.IRoleMenuService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 角色菜单关系表 服务实现类
 * </p>
 *
 * @author flyer
 * @since 2024-05-06
 */
@Service
public class RoleMenuServiceImpl extends ServiceImpl<IRoleMenuMapper, RoleMenu> implements IRoleMenuService {
    @Transactional
    @Override
    public boolean addRoleMenu(RoleMenuAddDTO addDTO) {
        if (addDTO.getRoleId() == null) {
            throw new BusinessException("请选择角色");
        }
        if (addDTO.getMenuIdList().size() == 0) {
            throw new BusinessException("请选择菜单");
        }
        // 物理删除菜单
        baseMapper.deleteByRoleIdList(Collections.singletonList(addDTO.getRoleId()));
        // 添加角色菜单
        RoleMenu roleMenu;
        for (Integer menuId : addDTO.getMenuIdList()) {
            roleMenu = new RoleMenu();
            roleMenu.setRoleId(addDTO.getRoleId());
            roleMenu.setMenuId(menuId);
            this.save(roleMenu);
        }
        return true;
    }

    @Override
    public List<Integer> getRoleMenuIdList(Integer roleId) {
        return this.lambdaQuery().eq(RoleMenu::getRoleId, roleId).list().stream().map(RoleMenu::getMenuId).collect(Collectors.toList());
    }

    @Override
    public boolean deleteByRoleIdList(List<Integer> roleIdList) {
        return baseMapper.deleteByRoleIdList(roleIdList);
    }
}
